#include <stdio.h>
#include <stdlib.h>
#include <err.h>
#include <limits.h>
#include <string.h>
#include <math.h>
#include <unistd.h>
#include <sys/time.h>

#include "util.h"

static void
usage(void)
{
	errx(1, "usage: randnum [-d delimstr] [-n] [-N num] "
	        "[-p prec] [-s seed] "
	        "[[min_val] max_val]");
}

int
main(int argc, char *argv[])
{
	int i, ch, s = 0, prec = 17, finalnl = 1;
	long seed, n = 1;
	double minv = 0.0, maxv = 1.0;
	char *delimstr = "\n";
	const char *errstr;
	struct timeval t1;

	if (pledge("stdio", NULL) == -1)
		err(2, "pledge");

	while ((ch = getopt(argc, argv, "d:nN:p:s:")) != -1) {
		switch (ch) {
		case 'd':
			delimstr = optarg;
			break;
		case 'n':
			finalnl = 0;
			break;
		case 'N':
			n = strtonum(optarg, 0, LONG_MAX, &errstr);
			if (errstr != NULL)
				errx(1, "bad num value, %s: %s", errstr, optarg);
			break;
		case 'p':
			prec = strtonum(optarg, 0, INT_MAX, &errstr);
			if (errstr != NULL)
				errx(1, "bad precision value, %s: %s", errstr, optarg);
			break;
		case 's':
			seed = strtonum(optarg, 0, INT_MAX, &errstr);
			if (errstr != NULL)
				errx(1, "bad seed value, %s: %s", errstr, optarg);
			break;
		default:
			usage();
		}
	}
	argc -= optind;
	argv += optind;

	if (argc > 2)
		usage();
	else if (argc == 2) {
		if (!sscanf(argv[0], "%lf", &minv))
			errx(1, "bad minv value: %s", argv[0]);
		if (!sscanf(argv[1], "%lf", &maxv))
			errx(1, "bad maxv value: %s", argv[1]);
	} else if (argc == 1)
		if (!sscanf(argv[0], "%lf", &maxv))
			errx(1, "bad maxv value: %s", argv[0]);

	if (s)
#ifdef __OpenBSD__
		srand48_deterministic(seed);
#else
		srand48(seed);
	else {
		gettimeofday(&t1, NULL);
		srand48(t1.tv_sec * t1.tv_usec); /* Caveat: identical seed for each ms */
	}
#endif

	for (i = 0; i < n; i++) {
		printf("%.*g", prec, drand48() * (maxv - minv) + minv);
		if (i < n - 1)
			fputs(delimstr, stdout);
	}
	if (finalnl)
		putchar('\n');

	return 0;
}
