#include <stdio.h>
#include <stdlib.h>
#include <err.h>
#include <limits.h>
#include <string.h>
#include <math.h>
#include <unistd.h>

#include "util.h"

static void
usage(void)
{
	errx(1, "usage: range [-b] [-d delimstr] [-e] [-l] [-n] [-N num] "
	     "[-p prec] [-s] [[min_val] max_val]");
}

int
main(int argc, char *argv[])
{
	int i, j, ch, n = 10, logrange = 0, openstart = 0, openend = 0,
		prec = 17, finalnl = 1, reportdx = 0;
	double minv = 0.0, maxv = 1.0, dx, val;
	const char *errstr;
	char *delimstr = "\n";

	if (pledge("stdio", NULL) == -1)
		err(2, "pledge");

	while ((ch = getopt(argc, argv, "bd:elnN:p:s")) != -1) {
		switch (ch) {
		case 'b':
			openstart = 1;
			break;
		case 'd':
			delimstr = optarg;
			break;
		case 'e':
			openend = 1;
			break;
		case 'l':
			logrange = 1;
			break;
		case 'n':
			finalnl = 0;
			break;
		case 'N':
			n = strtonum(optarg, 0, LONG_MAX, &errstr);
			if (errstr != NULL)
				errx(1, "bad num value, %s: %s", errstr, optarg);
			break;
		case 'p':
			prec = strtonum(optarg, -10, INT_MAX, &errstr);
			if (errstr != NULL)
				errx(1, "bad precision value, %s: %s", errstr, optarg);
			break;
		case 's':
			reportdx = 1;
			break;
		default:
			usage();
		}
	}
	argc -= optind;
	argv += optind;
	if (argc > 2)
		usage();
	else if (argc == 2) {
		if (!sscanf(argv[0], "%lf", &minv))
			errx(1, "bad minv value: %s", argv[0]);
		if (!sscanf(argv[1], "%lf", &maxv))
			errx(1, "bad maxv value: %s", argv[1]);
	} else if (argc == 1)
		if (!sscanf(argv[0], "%lf", &maxv))
			errx(1, "bad maxv value: %s", argv[0]);

	if (logrange) {
		if (minv <= 0.0)
			errx(1, "minv is not positive: %g", minv);
		if (maxv <= 0.0)
			errx(1, "maxv is not positive: %g", maxv);
		minv = log10(minv);
		maxv = log10(maxv);
	}
	dx = (maxv - minv) / (n - 1 + openend + openstart);
	if (reportdx) {
		printf("%.*g", prec, dx);
		if (finalnl)
			putchar('\n');
		return 0;
	}
	for (i = 0; i < n; i++) {
		j = i + openstart;
		if (logrange)
			val = pow(10, minv + j * dx);
		else
			val = minv + j * dx;
		printf("%.*g", prec, val);
		if (i < n - 1)
			fputs(delimstr, stdout);
	}
	if (finalnl)
		putchar('\n');

	return 0;
}
