#include <stdio.h>
#include <stdlib.h>
#include <err.h>
#include <limits.h>
#include <string.h>
#include <unistd.h>

#include "util.h"

#define VALUESTR "@VAL@"

static void
usage(void)
{
	errx(1, "usage: rangetest [-n maxiter] cmd min_val max_val\n"
		"where cmd must contain the string '" VALUESTR "'");
}

static int
launch(char *cmd, char *cmd0, double val)
{
	char *c;

	if ((c = strstr(cmd0, VALUESTR)) == NULL)
		errx(1, VALUESTR " not found in cmd");
	if (strlcpy(cmd, cmd0, PATH_MAX) >= PATH_MAX)
		err(1, "cmd too long");
	sprintf(cmd + (c - cmd0), "%.17g%s >/dev/null",
	        val, c + strnlen(VALUESTR, sizeof(cmd)));
	if (system(cmd)) {
		fprintf(stderr, "%.17g\n", val);
		return 1;
	} else
		printf("%.17g\n", val);

	return 0;
}

static void
binary_search(char *cmd, char *cmd0, double minv, double maxv, int maxiter)
{
	int minfail, maxfail, iter = 0;
	double val;

	minfail = launch(cmd, cmd0, minv);
	maxfail = launch(cmd, cmd0, maxv);
	if (minfail && maxfail)
		errx(2, "both min_val and max_val runs errored");
	else if (!minfail && !maxfail)
		errx(3, "both min_val and max_val ran successfully");

	while (minv <= maxv && iter < maxiter) {
		val = minv + 0.5 * (maxv - minv);
		if (launch(cmd, cmd0, val)) {
			if (maxfail) {
				maxv = val;
				maxfail = 1;
			} else {
				minv = val;
			}
		} else {
			if (maxfail) {
				minv = val;
			} else {
				maxv = val;
				maxfail = 0;
			}
		}
		iter++;
	}
}

int
main(int argc, char *argv[])
{
	int ch, maxiter = 10;
	double minv, maxv;
	char cmd0[PATH_MAX] = "", cmd[PATH_MAX] = "";
	const char *errstr;

	while ((ch = getopt(argc, argv, "N:")) != -1) {
		switch (ch) {
		case 'N':
			maxiter = strtonum(optarg, 0, INT_MAX, &errstr);
			if (errstr != NULL)
				errx(1, "bad maxiter value, %s: %s", errstr, optarg);
			break;
		default:
			usage();
		}
	}
	argc -= optind;
	argv += optind;
	if (argc == 3) {
		if (strlcpy(cmd0, argv[0], sizeof(cmd0)) >= sizeof(cmd0))
			err(1, "cmd too long");
		minv = atof(argv[1]);
		maxv = atof(argv[2]);
		if (minv >= maxv)
			errx(1, "min_val must be smaller than max_val");
		binary_search(cmd, cmd0, minv, maxv, maxiter);
	} else
		usage();

	return 0;
}
