#include <err.h>
#include <stdio.h>
#include <stdlib.h>
#include <unistd.h>
#include <math.h>
#include <limits.h>
#include <unistd.h>

#include "util.h"

static void
usage(void)
{
	errx(1, "usage: stdvar [-d delimstr] [-n] [-p prec] [-u]");
}

int
main(int argc, char *argv[])
{
	int ch, prec = 17, finalnl = 1;
	size_t i, j, nf = 0, nr = 0, correction = 1;
	double *means = NULL, *stdvars = NULL, **vals = NULL;
	const char *errstr;
	char *delimstr = "\t";

	if (pledge("stdio", NULL) == -1)
		err(2, "pledge");

	while ((ch = getopt(argc, argv, "d:np:u")) != -1) {
		switch (ch) {
		case 'd':
			delimstr = optarg;
			break;
		case 'n':
			finalnl = 0;
			break;
		case 'p':
			prec = strtonum(optarg, -10, INT_MAX, &errstr);
			if (errstr != NULL)
				errx(1, "bad precision value, %s: %s", errstr, optarg);
			break;
		case 'u':
			correction = 0;
			break;
		default:
			usage();
		}
	}
	/*argc -= optind;*/
	/*argv += optind;*/

	nr = fscanmatrix(stdin, &vals, &nf);

	if (!(means = calloc(nf, sizeof(double))) ||
	    !(stdvars = calloc(nf, sizeof(double))))
		err(1, "calloc");

	for (i = 0; i < nf; i++) {
		means[i] = 0.0;
		for (j = 0; j < nr; j++)
			means[i] += vals[j][i];
		means[i] /= (double)nr;
	}

	for (i = 0; i < nf; i++) {
		stdvars[i] = 0.0;
		for (j = 0; j < nr; j++)
			stdvars[i] += pow(vals[j][i] - means[i], 2.0);
		stdvars[i] /= (double)(nr - correction);
	}

	printfarr(delimstr, prec, stdvars, nf);
	if (finalnl)
		putchar('\n');

	free(means);
	free(stdvars);
	for (i = 0; i < nr; i++)
		free(vals[i]);
	free(vals);

	return 0;
}
