#include <err.h>
#include <stdio.h>
#include <stdlib.h>
#include <unistd.h>
#include <limits.h>

#include "util.h"

static void
usage(void)
{
	errx(1, "usage: sum [-d delimstr] [-n] [-p prec]");
}

int
main(int argc, char *argv[])
{
	int ch, prec = 17, finalnl = 1;
	size_t i = 0, nf = 0, nr = 0, linesize = 0;
	char *line = NULL, *data = NULL, *delimstr = "\t";
	const char *errstr;
	double val, *vals = NULL;

	if (pledge("stdio", NULL) == -1)
		err(2, "pledge");

	while ((ch = getopt(argc, argv, "d:np:")) != -1) {
		switch (ch) {
		case 'd':
			delimstr = optarg;
			break;
		case 'n':
			finalnl = 0;
			break;
		case 'p':
			prec = strtonum(optarg, 0, INT_MAX, &errstr);
			if (errstr != NULL)
				errx(1, "bad precision value, %s: %s", errstr, optarg);
			break;
		default:
			usage();
		}
	}
	argc -= optind;
	/*argv += optind;*/
	if (argc > 0)
		usage();

	while (getline(&line, &linesize, stdin) > 0) {
		if (nr == 0)
			if ((nf = allocarr(&vals, line, linesize)) == 0)
				errx(1, "no fields in input");
		data = line;
		for (i = 0; i < nf; i++) {
			if (!scannextval(&data, &val))
				errx(1, "could not parse line %ld, field %ld", nr + 1, i + 1);
			if (nr == 0)
				vals[i] = 0.0;
			vals[i] += val;
		}
		nr++;
	}
	printfarr(delimstr, prec, vals, nf);
	if (finalnl)
		putchar('\n');

	free(line);
	free(vals);

	return 0;
}
