#!/bin/sh

strtest()
{
	if ! test "$2" = "$3"
	then
		printf 'FAIL: %s: %s != %s\n' "$1" "$2" "$3"
	fi
}

strtest range_man "0.20000000000000001
0.40000000000000002
0.60000000000000009
0.80000000000000004" "$(./range -be -N 4)"
strtest range_man "1 2 3" "$(./range -d' ' -N 3 1 3)"
strtest range_man "1
10
100" "$(./range -l -N 3 0 2)"
strtest range_man "-2
-1.5
-1" "$(./range -N 3 -- -2 -1)"
strtest range "0
0.1111111111111111
0.22222222222222221
0.33333333333333331
0.44444444444444442
0.55555555555555558
0.66666666666666663
0.77777777777777768
0.88888888888888884
1" "$(./range)"
strtest range "0 0.33333333333333331 0.66666666666666663 1" "$(./range -n -d' ' -N 4)"
strtest range "0 0.33 0.67 1" "$(./range -n -p 2 -d' ' -N 4)"
strtest range "0.25 0.5 0.75 1" "$(./range -b -p 2 -d' ' -N 4)"
strtest range "0 0.25 0.5 0.75" "$(./range -e -p 2 -d' ' -N 4)"
strtest range_man "0
0.33333333333333331
0.66666666666666663
1" "$(./range -N 4)"

strtest max_man "4	5	6" "$(printf '1\t2\t3\n4\t5\t6\n' | ./max)"
strtest max "4,5,6" "$(printf '1\t2\t3\n4\t5\t6\n' | ./max -d',')"
strtest max "1" "$(./range | ./max)"
strtest max "0.90909090909090917" "$(./range -be | ./max)"
strtest max "0.909" "$(./range -be | ./max -p 3 )"

strtest mean_man "2.5	3.5	4.5" "$(printf '1\t2\t3\n4\t5\t6\n' | ./mean)"
strtest mean "2.5,3.5,4.5" "$(printf '1\t2\t3\n4\t5\t6\n' | ./mean -d',')"
strtest mean "0.5" "$(./range | ./mean)"
strtest mean "0.5" "$(./range -be | ./mean)"
strtest mean "0.5" "$(./range -be | ./mean -p 3 )"

strtest min_man "1	2	3" "$(printf '1\t2\t3\n4\t5\t6\n' | ./min)"
strtest min "1,2,3" "$(printf '1\t2\t3\n4\t5\t6\n' | ./min -d',')"
strtest min "0" "$(./range | ./min)"
strtest min "0.090909090909090912" "$(./range -be | ./min)"
strtest min "0.0909" "$(./range -be | ./min -p 3 )"

strtest rangetest_man "0
1.25
1.40625
1.484375
1.494140625" "$(./rangetest './condfail @VAL@' 0.0 10.0 2>/dev/null)"

strtest rangetest_man "10
5
2.5
1.875
1.5625
1.5234375
1.50390625" "$(./rangetest './condfail @VAL@' 0.0 10.0 2>&1 >/dev/null)"

strtest stddev_man "2.1908902300206643" "$(printf '10\n8\n10\n8\n8\n4\n' | ./stddev)"
strtest stddev_man "2" "$(printf '10\n8\n10\n8\n8\n4\n' | ./stddev -u)"
strtest stddev "2.2" "$(printf '10\n8\n10\n8\n8\n4\n' | ./stddev -p 2)"
strtest stddev "2.1213203435596424	2.1213203435596424	2.1213203435596424" "$(printf '1\t2\t3\n4\t5\t6\n' | ./stddev)"
strtest stddev "2.1	2.1	2.1" "$(printf '1\t2\t3\n4\t5\t6\n' | ./stddev -p 2)"
strtest stddev "2.1,2.1,2.1" "$(printf '1\t2\t3\n4\t5\t6\n' | ./stddev -p 2 -d',' )"

strtest stdvar_man "4.7999999999999998" "$(printf '10\n8\n10\n8\n8\n4\n' | ./stdvar)"
strtest stdvar_man "4" "$(printf '10\n8\n10\n8\n8\n4\n' | ./stdvar -u)"
strtest stdvar "4.8" "$(printf '10\n8\n10\n8\n8\n4\n' | ./stdvar -p 2)"
strtest stdvar "4.5	4.5	4.5" "$(printf '1\t2\t3\n4\t5\t6\n' | ./stdvar)"
strtest stdvar "4.5	4.5	4.5" "$(printf '1\t2\t3\n4\t5\t6\n' | ./stdvar -p 2)"
strtest stdvar "4.5,4.5,4.5" "$(printf '1\t2\t3\n4\t5\t6\n' | ./stdvar -p 2 -d',' )"

strtest sum_man "5	7	9" "$(printf '1\t2\t3\n4\t5\t6\n' | ./sum)"
strtest sum "48" "$(printf '10\n8\n10\n8\n8\n4\n' | ./sum)"
strtest sum "5e+01" "$(printf '10\n8\n10\n8\n8\n4\n' | ./sum -p 1)"
strtest sum "5	7	9" "$(printf '1\t2\t3\n4\t5\t6\n' | ./sum)"
strtest sum "5,7,9" "$(printf '1\t2\t3\n4\t5\t6\n' | ./sum -d',' )"

strtest transpose_man "1	4
2	5
3	6" "$(printf '1\t2\t3\n4\t5\t6\n' | ./transpose)"
