#include <err.h>
#include <stdlib.h>
#include <stdio.h>
#include "util.h"

size_t
allocarr(double **arr, const char *str, size_t maxlen)
{
	size_t i, nf = 0;

	if (maxlen > 0)
		nf = 1;
	for (i = 0; i < maxlen && str[i] != '\0'; i++)
		if (str[i] == DELIM)
			nf++;
	if (!(*arr = calloc(nf, sizeof(double))))
		err(1, "calloc");

	return nf;
}

int
scannextval(char **str, double *val)
{
	int offset;

	if (sscanf(*str, "%lg%n", val, &offset) != 1)
		return 0;
	*str += offset;

	return 1;
}

void
printarr(double *arr, size_t len)
{
	size_t i;

	for (i = 0; i < len; i++) {
		printf("%.17g", arr[i]);
		if (i < len)
			printf(DELIMSTR);
	}
	putchar('\n');
}

void
printfarr(char *delimstr, int prec, double *arr, size_t len)
{
	size_t i;

	for (i = 0; i < len; i++) {
		printf("%.*g", prec, arr[i]);
		if (i < len - 1)
			fputs(delimstr, stdout);
	}
}

size_t
fscanmatrix(FILE *stream, double ***arr, size_t *nf)
{
	size_t i, nr = 0, linesize = 0;
	char *line = NULL, *data = NULL;
	double val;

	*arr = calloc(1, sizeof(double *));

	while (getline(&line, &linesize, stream) > 0) {
		if (nr > 0)
			if (!(*arr = xreallocarray(*arr, nr + 1, sizeof(double *))))
				err(1, "reallocarray");
		if ((*nf = allocarr(&(*arr)[nr], line, linesize)) == 0)
			errx(1, "no fields in input");
		data = line;
		for (i = 0; i < *nf; i++) {
			if (!scannextval(&data, &val))
				errx(1, "could not parse line %ld, field %ld", nr + 1, i + 1);
			(*arr)[nr][i] = val;
		}
		nr++;
	}

	free(line);

	return nr;
}